# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface

import enum


class ZwpTabletPadRingV2(Interface):
    """Pad ring

    A circular interaction area, such as the touch ring on the Wacom Intuos Pro
    series tablets.

    Events on a ring are logically grouped by the wl_tablet_pad_ring.frame
    event.
    """
    name = "zwp_tablet_pad_ring_v2"
    version = 1

    source = enum.Enum("source", {
        "finger": 1,
    })


@ZwpTabletPadRingV2.request("su", [None, None])
def set_feedback(self, description, serial):
    """Set compositor feedback

    Request that the compositor use the provided feedback string associated
    with this ring. This request should be issued immediately after a
    wp_tablet_pad_group.mode_switch event from the corresponding group is
    received, or whenever the ring is mapped to a different action. See
    wp_tablet_pad_group.mode_switch for more details.

    Clients are encouraged to provide context-aware descriptions for the
    actions associated with the ring; compositors may use this information to
    offer visual feedback about the button layout (eg. on-screen displays).

    The provided string 'description' is a UTF-8 encoded string to be
    associated with this ring, and is considered user-visible; general
    internationalization rules apply.

    The serial argument will be that of the last
    wp_tablet_pad_group.mode_switch event received for the group of this ring.
    Requests providing other serials than the most recent one will be ignored.

    :param description:
        ring description
    :type description: `string`
    :param serial:
        serial of the mode switch event
    :type serial: `uint`
    """
    self._marshal(0, description, serial)


@ZwpTabletPadRingV2.request("", [])
def destroy(self):
    """Destroy the ring object

    This destroys the client's resource for this ring object.
    """
    self._marshal(1)
    self._destroy()


@ZwpTabletPadRingV2.event("u", [None])
def source(self, source):
    """Ring event source

    Source information for ring events.

    This event does not occur on its own. It is sent before a
    wp_tablet_pad_ring.frame event and carries the source information for all
    events within that frame.

    The source specifies how this event was generated. If the source is
    wp_tablet_pad_ring.source.finger, a wp_tablet_pad_ring.stop event will be
    sent when the user lifts the finger off the device.

    This event is optional. If the source is unknown for an interaction, no
    event is sent.

    :param source:
        the event source
    :type source: `uint`
    """
    self._post_event(0, source)


@ZwpTabletPadRingV2.event("f", [None])
def angle(self, degrees):
    """Angle changed

    Sent whenever the angle on a ring changes.

    The angle is provided in degrees clockwise from the logical north of the
    ring in the pad's current rotation.

    :param degrees:
        the current angle in degrees
    :type degrees: `fixed`
    """
    self._post_event(1, degrees)


@ZwpTabletPadRingV2.event("", [])
def stop(self):
    """Interaction stopped

    Stop notification for ring events.

    For some wp_tablet_pad_ring.source types, a wp_tablet_pad_ring.stop event
    is sent to notify a client that the interaction with the ring has
    terminated. This enables the client to implement kinetic scrolling. See the
    wp_tablet_pad_ring.source documentation for information on when this event
    may be generated.

    Any wp_tablet_pad_ring.angle events with the same source after this event
    should be considered as the start of a new interaction.
    """
    self._post_event(2)


@ZwpTabletPadRingV2.event("u", [None])
def frame(self, time):
    """End of a ring event sequence

    Indicates the end of a set of ring events that logically belong together. A
    client is expected to accumulate the data in all events within the frame
    before proceeding.

    All wp_tablet_pad_ring events before a wp_tablet_pad_ring.frame event
    belong logically together. For example, on termination of a finger
    interaction on a ring the compositor will send a wp_tablet_pad_ring.source
    event, a wp_tablet_pad_ring.stop event and a wp_tablet_pad_ring.frame
    event.

    A wp_tablet_pad_ring.frame event is sent for every logical event group,
    even if the group only contains a single wp_tablet_pad_ring event.
    Specifically, a client may get a sequence: angle, frame, angle, frame, etc.

    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    """
    self._post_event(3, time)


ZwpTabletPadRingV2._gen_c()



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland import ffi, lib


# void (*wl_notify_func_t)(struct wl_listener *listener, void *data);
@ffi.def_extern()
def notify_func(listener_ptr, data):
    # basically a `container_of` macro, but using cffi, get the
    # wl_listener_container for the given listener
    container = ffi.cast(
        "struct wl_listener_container *",
        ffi.cast("char*", listener_ptr) - ffi.offsetof("struct wl_listener_container", "destroy_listener")
    )
    listener = ffi.from_handle(container.handle)

    callback = listener["notify"]
    callback(listener["link"])


class Listener:
    """A single listener for Wayland signals

    Provides the means to listen for `wl_listener` signal notifications.  Many
    Wayland objects use `wl_listener` for notification of significant events
    like object destruction.

    Clients should create :class:`Listener` objects manually and can register
    them as listeners to objects destroy events using the object's
    ``.add_destroy_listener()`` method.  A listener can only listen to one
    signal at a time.

    :param function: callback function for the Listener
    :type function: callable
    """
    def __init__(self, function):
        self._callback_info = {
            "notify": function,
            "link": None
        }
        self._handle = ffi.new_handle(self._callback_info)

        # we need a way to get this Python object from the `struct
        # wl_listener*`, so we put the pointer in a container struct that
        # contains both the wl_listener and a pointer to our ffi handle
        self.container = ffi.new("struct wl_listener_container *")
        self.container.handle = self._handle

        self._ptr = ffi.addressof(self.container.destroy_listener)
        self._ptr.notify = lib.notify_func

    @property
    def link(self):
        return self._callback_info["link"]

    @link.setter
    def link(self, value):
        self._callback_info["link"] = value

    def remove(self):
        """Remove the listener"""
        if self.link:
            lib.wl_list_remove(ffi.addressof(self._ptr.link))
            self.link = None


class Signal:
    """A source of a type of observable event

    Signals are recognized points where significant events can be observed.
    Compositors as well as the server can provide signals. Observers are
    wl_listener's that are added through #wl_signal_add. Signals are emitted
    using #wl_signal_emit, which will invoke all listeners until that listener
    is removed by wl_list_remove() (or whenever the signal is destroyed).
    """
    def __init__(self):
        self._ptr = ffi.new("struct wl_listener *")

        lib.wl_signal_init(self._ptr)

    def add(self, listener):
        """Add the specified listener to this signal

        :param listener: The listener to add
        :type listener: :class:`Listener`
        """
        lib.wl_signal_add(self._ptr, listener._ptr)

    def emit(self, data=None):
        """Emits this signal, notifying all registered listeners

        :param data: The data that will be emitted with the signal
        """
        if data:
            data_ptr = ffi.new_handle(data)
        else:
            data_ptr = ffi.NULL
        lib.wl_signal.emit(self._ptr, data_ptr)



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland import ffi, lib
from pywayland.dispatcher import dispatcher_to_object
from pywayland.utils import ensure_valid

import functools


def _client_destroy(display, cdata):
    # do nothing if the display is already destroyed
    if display._ptr is None:
        return
    lib.wl_client_destroy(cdata)


class Client:
    """Create a client for the given file descriptor

    Given a file descriptor corresponding to one end of a socket, create a
    client struct and add the new client to the compositors client list.  At
    that point, the client is initialized and ready to run, as if the client
    had connected to the servers listening socket.

    The other end of the socket can be passed to
    :meth:`~pywayland.client.Display.connect()` on the client side or used with
    the WAYLAND_SOCKET environment variable on the client side.

    :param display: The display object
    :type display: :class:`Display`
    :param fd: The file descriptor for the socket to the client
    :type fd: `int`
    """
    def __init__(self, display, fd):
        if display._ptr is None:
            raise ValueError("Display has been destroyed")

        ptr = lib.wl_client_create(display._ptr, fd)

        destructor = functools.partial(_client_destroy, display)
        self._ptr = ffi.gc(ptr, destructor)
        self._display = display

    def destroy(self):
        """Destroy the client"""
        if self._ptr is not None:
            # destroy the client and remove the destructor
            _client_destroy(self._display, self._ptr)
            ffi.gc(self._ptr, None)
            self._ptr = None
            self._display = None

    @ensure_valid
    def flush(self):
        """Flush pending events to the client

        Events sent to clients are queued in a buffer and written to the socket
        later - typically when the compositor has handled all requests and goes
        back to block in the event loop.  This function flushes all queued up
        events for a client immediately.
        """
        lib.wl_client_flush(self._ptr)

    @ensure_valid
    def add_destroy_listener(self, listener):
        """Add a listener for the destroy signal

        :param listener: The listener object
        :type listener: :class:`~pywayland.server.Listener`
        """
        lib.wl_client_add_destroy_listener(self._ptr, listener._ptr)
        listener.link = self

    @ensure_valid
    def get_object(self, id):
        """Look up an object in the client name space

        This looks up an object in the client object name space by its object
        ID.

        :param id: The object id
        :type id: `int`
        :returns: The object, or ``None`` if there is not object for the given
                  ID
        """

        res_ptr = lib.wl_client_get_object(self._ptr, id)
        # If the object doesn't exist, this returns NULL, and asking for
        # forgiveness doesn't work, becuase it will seg fault
        if res_ptr == ffi.NULL:
            return
        # we attach the handle to resources, so first get the dispatcher, then resolve it's corresponding resource
        dispatcher_handle = lib.wl_resource_get_user_data(res_ptr)
        dispatcher = ffi.from_handle(dispatcher_handle)
        return dispatcher_to_object.get(dispatcher)



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .wl_surface import WlSurface

import enum


class WlPointer(Interface):
    """Pointer input device

    The :class:`WlPointer` interface represents one or more input devices, such
    as mice, which control the pointer location and pointer_focus of a seat.

    The :class:`WlPointer` interface generates motion, enter and leave events
    for the surfaces that the pointer is located over, and button and axis
    events for button presses, button releases and scrolling.
    """
    name = "wl_pointer"
    version = 7

    error = enum.Enum("error", {
        "role": 0,
    })

    button_state = enum.Enum("button_state", {
        "released": 0,
        "pressed": 1,
    })

    axis = enum.Enum("axis", {
        "vertical_scroll": 0,
        "horizontal_scroll": 1,
    })

    axis_source = enum.Enum("axis_source", {
        "wheel": 0,
        "finger": 1,
        "continuous": 2,
        "wheel_tilt": 3,
    })


@WlPointer.request("u?oii", [None, WlSurface, None, None])
def set_cursor(self, serial, surface, hotspot_x, hotspot_y):
    """Set the pointer surface

    Set the pointer surface, i.e., the surface that contains the pointer image
    (cursor). This request gives the surface the role of a cursor. If the
    surface already has another role, it raises a protocol error.

    The cursor actually changes only if the pointer focus for this device is
    one of the requesting client's surfaces or the surface parameter is the
    current pointer surface. If there was a previous surface set with this
    request it is replaced. If surface is NULL, the pointer image is hidden.

    The parameters hotspot_x and hotspot_y define the position of the pointer
    surface relative to the pointer location. Its top-left corner is always at
    (x, y) - (hotspot_x, hotspot_y), where (x, y) are the coordinates of the
    pointer location, in surface-local coordinates.

    On surface.attach requests to the pointer surface, hotspot_x and hotspot_y
    are decremented by the x and y parameters passed to the request. Attach
    must be confirmed by :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface..commit>` as usual.

    The hotspot can also be updated by passing the currently set pointer
    surface to this request with new values for hotspot_x and hotspot_y.

    The current and pending input regions of the
    :class:`~pywayland.protocol.wayland.WlSurface` are cleared, and
    :func:`WlSurface.set_input_region()
    <pywayland.protocol.wayland.WlSurface..set_input_region>` is ignored until
    the :class:`~pywayland.protocol.wayland.WlSurface` is no longer used as the
    cursor. When the use as a cursor ends, the current and pending input
    regions become undefined, and the
    :class:`~pywayland.protocol.wayland.WlSurface` is unmapped.

    :param serial:
        serial number of the enter event
    :type serial: `uint`
    :param surface:
        pointer surface
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface` or `None`
    :param hotspot_x:
        surface-local x coordinate
    :type hotspot_x: `int`
    :param hotspot_y:
        surface-local y coordinate
    :type hotspot_y: `int`
    """
    self._marshal(0, serial, surface, hotspot_x, hotspot_y)


@WlPointer.request("3", [])
def release(self):
    """Release the pointer object

    Using this request a client can tell the server that it is not going to use
    the pointer object anymore.

    This request destroys the pointer proxy object, so clients must not call
    wl_pointer_destroy() after using this request.
    """
    self._marshal(1)
    self._destroy()


@WlPointer.event("uoff", [None, WlSurface, None, None])
def enter(self, serial, surface, surface_x, surface_y):
    """Enter event

    Notification that this seat's pointer is focused on a certain surface.

    When a seat's focus enters a surface, the pointer image is undefined and a
    client should respond to this event by setting an appropriate pointer image
    with the set_cursor request.

    :param serial:
        serial number of the enter event
    :type serial: `uint`
    :param surface:
        surface entered by the pointer
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    :param surface_x:
        surface-local x coordinate
    :type surface_x: `fixed`
    :param surface_y:
        surface-local y coordinate
    :type surface_y: `fixed`
    """
    self._post_event(0, serial, surface, surface_x, surface_y)


@WlPointer.event("uo", [None, WlSurface])
def leave(self, serial, surface):
    """Leave event

    Notification that this seat's pointer is no longer focused on a certain
    surface.

    The leave notification is sent before the enter notification for the new
    focus.

    :param serial:
        serial number of the leave event
    :type serial: `uint`
    :param surface:
        surface left by the pointer
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    """
    self._post_event(1, serial, surface)


@WlPointer.event("uff", [None, None, None])
def motion(self, time, surface_x, surface_y):
    """Pointer motion event

    Notification of pointer location change. The arguments surface_x and
    surface_y are the location relative to the focused surface.

    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    :param surface_x:
        surface-local x coordinate
    :type surface_x: `fixed`
    :param surface_y:
        surface-local y coordinate
    :type surface_y: `fixed`
    """
    self._post_event(2, time, surface_x, surface_y)


@WlPointer.event("uuuu", [None, None, None, None])
def button(self, serial, time, button, state):
    """Pointer button event

    Mouse button click and release notifications.

    The location of the click is given by the last motion or enter event. The
    time argument is a timestamp with millisecond granularity, with an
    undefined base.

    The button is a button code as defined in the Linux kernel's linux/input-
    event-codes.h header file, e.g. BTN_LEFT.

    Any 16-bit button code value is reserved for future additions to the
    kernel's event code list. All other button codes above 0xFFFF are currently
    undefined but may be used in future versions of this protocol.

    :param serial:
        serial number of the button event
    :type serial: `uint`
    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    :param button:
        button that produced the event
    :type button: `uint`
    :param state:
        physical state of the button
    :type state: `uint`
    """
    self._post_event(3, serial, time, button, state)


@WlPointer.event("uuf", [None, None, None])
def axis(self, time, axis, value):
    """Axis event

    Scroll and other axis notifications.

    For scroll events (vertical and horizontal scroll axes), the value
    parameter is the length of a vector along the specified axis in a
    coordinate space identical to those of motion events, representing a
    relative movement along the specified axis.

    For devices that support movements non-parallel to axes multiple axis
    events will be emitted.

    When applicable, for example for touch pads, the server can choose to emit
    scroll events where the motion vector is equivalent to a motion event
    vector.

    When applicable, a client can transform its content relative to the scroll
    distance.

    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    :param axis:
        axis type
    :type axis: `uint`
    :param value:
        length of vector in surface-local coordinate space
    :type value: `fixed`
    """
    self._post_event(4, time, axis, value)


@WlPointer.event("5", [])
def frame(self):
    """End of a pointer event sequence

    Indicates the end of a set of events that logically belong together. A
    client is expected to accumulate the data in all events within the frame
    before proceeding.

    All :class:`WlPointer` events before a :func:`WlPointer.frame()` event
    belong logically together. For example, in a diagonal scroll motion the
    compositor will send an optional :func:`WlPointer.axis_source()` event, two
    :func:`WlPointer.axis()` events (horizontal and vertical) and finally a
    :func:`WlPointer.frame()` event. The client may use this information to
    calculate a diagonal vector for scrolling.

    When multiple :func:`WlPointer.axis()` events occur within the same frame,
    the motion vector is the combined motion of all events. When a
    :func:`WlPointer.axis()` and a :func:`WlPointer.axis_stop()` event occur
    within the same frame, this indicates that axis movement in one axis has
    stopped but continues in the other axis. When multiple
    :func:`WlPointer.axis_stop()` events occur within the same frame, this
    indicates that these axes stopped in the same instance.

    A :func:`WlPointer.frame()` event is sent for every logical event group,
    even if the group only contains a single :class:`WlPointer` event.
    Specifically, a client may get a sequence: motion, frame, button, frame,
    axis, frame, axis_stop, frame.

    The :func:`WlPointer.enter()` and :func:`WlPointer.leave()` events are
    logical events generated by the compositor and not the hardware. These
    events are also grouped by a :func:`WlPointer.frame()`. When a pointer
    moves from one surface to another, a compositor should group the
    :func:`WlPointer.leave()` event within the same :func:`WlPointer.frame()`.
    However, a client must not rely on :func:`WlPointer.leave()` and
    :func:`WlPointer.enter()` being in the same :func:`WlPointer.frame()`.
    Compositor-specific policies may require the :func:`WlPointer.leave()` and
    :func:`WlPointer.enter()` event being split across multiple
    :func:`WlPointer.frame()` groups.
    """
    self._post_event(5)


@WlPointer.event("5u", [None])
def axis_source(self, axis_source):
    """Axis source event

    Source information for scroll and other axes.

    This event does not occur on its own. It is sent before a
    :func:`WlPointer.frame()` event and carries the source information for all
    events within that frame.

    The source specifies how this event was generated. If the source is
    :func:`WlPointer.axis_source()`.finger, a :func:`WlPointer.axis_stop()`
    event will be sent when the user lifts the finger off the device.

    If the source is :func:`WlPointer.axis_source()`.wheel,
    :func:`WlPointer.axis_source()`.wheel_tilt or
    :func:`WlPointer.axis_source()`.continuous, a :func:`WlPointer.axis_stop()`
    event may or may not be sent. Whether a compositor sends an axis_stop event
    for these sources is hardware-specific and implementation-dependent;
    clients must not rely on receiving an axis_stop event for these scroll
    sources and should treat scroll sequences from these scroll sources as
    unterminated by default.

    This event is optional. If the source is unknown for a particular axis
    event sequence, no event is sent. Only one :func:`WlPointer.axis_source()`
    event is permitted per frame.

    The order of :func:`WlPointer.axis_discrete()` and
    :func:`WlPointer.axis_source()` is not guaranteed.

    :param axis_source:
        source of the axis event
    :type axis_source: `uint`
    """
    self._post_event(6, axis_source)


@WlPointer.event("5uu", [None, None])
def axis_stop(self, time, axis):
    """Axis stop event

    Stop notification for scroll and other axes.

    For some :func:`WlPointer.axis_source()` types, a
    :func:`WlPointer.axis_stop()` event is sent to notify a client that the
    axis sequence has terminated. This enables the client to implement kinetic
    scrolling. See the :func:`WlPointer.axis_source()` documentation for
    information on when this event may be generated.

    Any :func:`WlPointer.axis()` events with the same axis_source after this
    event should be considered as the start of a new axis motion.

    The timestamp is to be interpreted identical to the timestamp in the
    :func:`WlPointer.axis()` event. The timestamp value may be the same as a
    preceding :func:`WlPointer.axis()` event.

    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    :param axis:
        the axis stopped with this event
    :type axis: `uint`
    """
    self._post_event(7, time, axis)


@WlPointer.event("5ui", [None, None])
def axis_discrete(self, axis, discrete):
    """Axis click event

    Discrete step information for scroll and other axes.

    This event carries the axis value of the :func:`WlPointer.axis()` event in
    discrete steps (e.g. mouse wheel clicks).

    This event does not occur on its own, it is coupled with a
    :func:`WlPointer.axis()` event that represents this axis value on a
    continuous scale. The protocol guarantees that each axis_discrete event is
    always followed by exactly one axis event with the same axis number within
    the same :func:`WlPointer.frame()`. Note that the protocol allows for other
    events to occur between the axis_discrete and its coupled axis event,
    including other axis_discrete or axis events.

    This event is optional; continuous scrolling devices like two-finger
    scrolling on touchpads do not have discrete steps and do not generate this
    event.

    The discrete value carries the directional information. e.g. a value of -2
    is two steps towards the negative direction of this axis.

    The axis number is identical to the axis number in the associated axis
    event.

    The order of :func:`WlPointer.axis_discrete()` and
    :func:`WlPointer.axis_source()` is not guaranteed.

    :param axis:
        axis type
    :type axis: `uint`
    :param discrete:
        number of steps
    :type discrete: `int`
    """
    self._post_event(8, axis, discrete)


WlPointer._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2012, 2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlSurface
from .zwp_input_panel_surface_v1 import ZwpInputPanelSurfaceV1


class ZwpInputPanelV1(Interface):
    """Interface for implementing keyboards

    Only one client can bind this interface at a time.
    """
    name = "zwp_input_panel_v1"
    version = 1


@ZwpInputPanelV1.request("no", [ZwpInputPanelSurfaceV1, WlSurface])
def get_input_panel_surface(self, surface):
    """get_input_panel_surface

    :param surface:
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    :returns:
        :class:`~pywayland.protocol.input_method_unstable_v1.ZwpInputPanelSurfaceV1`
    """
    id = self._marshal_constructor(0, ZwpInputPanelSurfaceV1, surface)
    return id


ZwpInputPanelV1._gen_c()



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland import ffi, lib

import re
from weakref import WeakKeyDictionary

weakkeydict = WeakKeyDictionary()  # type: WeakKeyDictionary

re_arg = re.compile(r"(\??)([uifsonah])")


class Message:
    """Wrapper class for `wl_message` structs

    Base class that correspond to the methods defined on an interface in the
    wayland.xml protocol, and are generated by the scanner.  Subclasses specify
    the type of method, whether it is a server-side or client-side method.

    :param func: The function that is represented by the message
    :type func: `function`
    :param signature: The signature of the arguments of the message
    :type signature: `string`
    :param types:  List of the types of any objects included in the argument
                   list, None if otherwise.
    :type types: `list`
    """
    def __init__(self, func, signature, types):
        self.py_func = func

        self.name = func.__name__.strip('_')
        self.signature = signature
        self.types = types

        self._ptr = ffi.new('struct wl_message *')
        self._ptr.name = name = ffi.new('char[]', self.name.encode())
        self._ptr.signature = signature = ffi.new('char[]', self.signature.encode())

        self._ptr.types = types = ffi.new('struct wl_interface* []', len(self.types))
        for i, type_ in enumerate(self.types):
            if type_:
                self._ptr.types[i] = type_._ptr
            else:
                self._ptr.types[i] = ffi.NULL

        weakkeydict[self._ptr] = (name, signature, types)

    def c_to_arguments(self, args_ptr):
        """Create a list of arguments

        Generate the arguments of the method from a CFFI cdata array of
        `wl_argument` structs that correspond to the arguments of the method as
        specified by the method signature.

        :param args_ptr: Input arguments
        :type args_ptr: cdata `union wl_argument []`
        :returns: list of args
        """
        args = []
        for i, sig_match in enumerate(re_arg.finditer(self.signature)):
            arg_ptr = args_ptr[i]
            null, sig = sig_match.groups()

            # Match numbers (int, unsigned, float, file descriptor)
            if sig == 'i':
                args.append(arg_ptr.i)
            elif sig == 'u':
                args.append(arg_ptr.u)
            elif sig == 'f':
                f = lib.wl_fixed_to_double(arg_ptr.f)
                args.append(f)
            elif sig == 'h':
                args.append(arg_ptr.h)
            # Match string
            elif sig == 's':
                if arg_ptr == ffi.NULL:
                    if not null:
                        raise Exception
                    args.append(None)
                else:
                    args.append(ffi.string(arg_ptr.s).decode())
            # Object
            elif sig == 'o':
                if arg_ptr.o == ffi.NULL:
                    if not null:
                        message = "Got null object parsing arguments for '{}' message, may already be destroyed".format(
                            self.name
                        )
                        raise RuntimeError(message)
                    args.append(None)
                else:
                    iface = self.types[i]
                    proxy_ptr = ffi.cast('struct wl_proxy *', arg_ptr.o)
                    obj = iface.proxy_class.registry.get(proxy_ptr)
                    if obj is None:
                        raise RuntimeError("Unable to get object for {}, was it garbage collected?".format(proxy_ptr))
                    args.append(obj)
            # New id
            elif sig == 'n':
                # TODO
                raise NotImplementedError
            # Array (i.e. buffer of bytes)
            elif sig == 'a':
                array_ptr = arg_ptr.a
                args.append(ffi.buffer(array_ptr.data, array_ptr.size)[:])

        return args

    def arguments_to_c(self, *args):
        """Create an array of `wl_argument` C structs

        Generate the CFFI cdata array of `wl_argument` structs that correspond
        to the arguments of the method as specified by the method signature.

        :param args: Input arguments
        :type args: `list`
        :returns: cdata `union wl_argument []` of args
        """
        nargs = len(re_arg.findall(self.signature))
        args_ptr = ffi.new('union wl_argument []', nargs)

        arg_iter = iter(args)
        refs = []
        for i, sig_match in enumerate(re_arg.finditer(self.signature)):
            null, sig = sig_match.groups()

            # New id (set to null for now, will be assigned on marshal)
            # Then, continue so we don't consume an arg
            if sig == 'n':
                args_ptr[i].o = ffi.NULL
                continue
            arg = next(arg_iter)
            # Match numbers (int, unsigned, float, file descriptor)
            if sig == 'i':
                args_ptr[i].i = arg
            elif sig == 'u':
                args_ptr[i].u = arg
            elif sig == 'f':
                if isinstance(arg, int):
                    f = lib.wl_fixed_from_int(arg)
                else:
                    f = lib.wl_fixed_from_double(arg)
                args_ptr[i].f = f
            elif sig == 'h':
                args_ptr[i].h = arg
            # Match string
            elif sig == 's':
                if arg is None:
                    if not null:
                        raise Exception
                    new_arg = ffi.NULL
                else:
                    new_arg = ffi.new('char []', arg.encode())
                    refs.append(new_arg)
                args_ptr[i].s = new_arg
            # Object
            elif sig == 'o':
                if arg is None:
                    if not null:
                        raise Exception
                    new_arg = ffi.NULL
                else:
                    new_arg = ffi.cast('struct wl_object *', arg._ptr)
                    refs.append(new_arg)
                args_ptr[i].o = new_arg
            # Array (i.e. buffer of bytes)
            elif sig == 'a':
                # TODO: this is a bit messy, we probably don't want to put everything in one buffer like this
                new_arg = ffi.new('struct wl_array *')
                new_data = ffi.new('void []', len(arg))
                new_arg.alloc = new_arg.size = len(arg)
                ffi.buffer(new_data)[:] = arg
                refs.append(new_arg)
                refs.append(new_data)

        if refs:
            weakkeydict[args_ptr] = refs

        return args_ptr



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2015, 2016 Red Hat
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlSeat
from .zwp_primary_selection_device_v1 import ZwpPrimarySelectionDeviceV1
from .zwp_primary_selection_source_v1 import ZwpPrimarySelectionSourceV1


class ZwpPrimarySelectionDeviceManagerV1(Interface):
    """X primary selection emulation

    The primary selection device manager is a singleton global object that
    provides access to the primary selection. It allows to create
    wp_primary_selection_source objects, as well as retrieving the per-seat
    wp_primary_selection_device objects.
    """
    name = "zwp_primary_selection_device_manager_v1"
    version = 1


@ZwpPrimarySelectionDeviceManagerV1.request("n", [ZwpPrimarySelectionSourceV1])
def create_source(self):
    """Create a new primary selection source

    Create a new primary selection source.

    :returns:
        :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionSourceV1`
    """
    id = self._marshal_constructor(0, ZwpPrimarySelectionSourceV1)
    return id


@ZwpPrimarySelectionDeviceManagerV1.request("no", [ZwpPrimarySelectionDeviceV1, WlSeat])
def get_device(self, seat):
    """Create a new primary selection device

    Create a new data device for a given seat.

    :param seat:
    :type seat: :class:`~pywayland.protocol.wayland.WlSeat`
    :returns:
        :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionDeviceV1`
    """
    id = self._marshal_constructor(1, ZwpPrimarySelectionDeviceV1, seat)
    return id


@ZwpPrimarySelectionDeviceManagerV1.request("", [])
def destroy(self):
    """Destroy the primary selection device manager

    Destroy the primary selection device manager.
    """
    self._marshal(2)
    self._destroy()


ZwpPrimarySelectionDeviceManagerV1._gen_c()



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .element import Element, Attribute


class Description(Element):
    attributes = [Attribute('summary', True)]
    pcdata = True

    def output(self, printer):
        printer.doc('"""{}'.format(self.summary.capitalize()))
        if self.text:
            printer()
            printer.docstring(self.text)



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .zwp_tablet_pad_ring_v2 import ZwpTabletPadRingV2
from .zwp_tablet_pad_strip_v2 import ZwpTabletPadStripV2


class ZwpTabletPadGroupV2(Interface):
    """A set of buttons, rings and strips

    A pad group describes a distinct (sub)set of buttons, rings and strips
    present in the tablet. The criteria of this grouping is usually positional,
    eg. if a tablet has buttons on the left and right side, 2 groups will be
    presented. The physical arrangement of groups is undisclosed and may change
    on the fly.

    Pad groups will announce their features during pad initialization. Between
    the corresponding wp_tablet_pad.group event and wp_tablet_pad_group.done,
    the pad group will announce the buttons, rings and strips contained in it,
    plus the number of supported modes.

    Modes are a mechanism to allow multiple groups of actions for every element
    in the pad group. The number of groups and available modes in each is
    persistent across device plugs. The current mode is user-switchable, it
    will be announced through the wp_tablet_pad_group.mode_switch event both
    whenever it is switched, and after wp_tablet_pad.enter.

    The current mode logically applies to all elements in the pad group,
    although it is at clients' discretion whether to actually perform different
    actions, and/or issue the respective .set_feedback requests to notify the
    compositor. See the wp_tablet_pad_group.mode_switch event for more details.
    """
    name = "zwp_tablet_pad_group_v2"
    version = 1


@ZwpTabletPadGroupV2.request("", [])
def destroy(self):
    """Destroy the pad object

    Destroy the wp_tablet_pad_group object. Objects created from this object
    are unaffected and should be destroyed separately.
    """
    self._marshal(0)
    self._destroy()


@ZwpTabletPadGroupV2.event("a", [None])
def buttons(self, buttons):
    """Buttons announced

    Sent on wp_tablet_pad_group initialization to announce the available
    buttons in the group. Button indices start at 0, a button may only be in
    one group at a time.

    This event is first sent in the initial burst of events before the
    wp_tablet_pad_group.done event.

    Some buttons are reserved by the compositor. These buttons may not be
    assigned to any wp_tablet_pad_group. Compositors may broadcast this event
    in the case of changes to the mapping of these reserved buttons. If the
    compositor happens to reserve all buttons in a group, this event will be
    sent with an empty array.

    :param buttons:
        buttons in this group
    :type buttons: `array`
    """
    self._post_event(0, buttons)


@ZwpTabletPadGroupV2.event("n", [ZwpTabletPadRingV2])
def ring(self, ring):
    """Ring announced

    Sent on wp_tablet_pad_group initialization to announce available rings. One
    event is sent for each ring available on this pad group.

    This event is sent in the initial burst of events before the
    wp_tablet_pad_group.done event.

    :param ring:
    :type ring: :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletPadRingV2`
    """
    self._post_event(1, ring)


@ZwpTabletPadGroupV2.event("n", [ZwpTabletPadStripV2])
def strip(self, strip):
    """Strip announced

    Sent on wp_tablet_pad initialization to announce available strips. One
    event is sent for each strip available on this pad group.

    This event is sent in the initial burst of events before the
    wp_tablet_pad_group.done event.

    :param strip:
    :type strip: :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletPadStripV2`
    """
    self._post_event(2, strip)


@ZwpTabletPadGroupV2.event("u", [None])
def modes(self, modes):
    """Mode-switch ability announced

    Sent on wp_tablet_pad_group initialization to announce that the pad group
    may switch between modes. A client may use a mode to store a specific
    configuration for buttons, rings and strips and use the
    wl_tablet_pad_group.mode_switch event to toggle between these
    configurations. Mode indices start at 0.

    Switching modes is compositor-dependent. See the
    wp_tablet_pad_group.mode_switch event for more details.

    This event is sent in the initial burst of events before the
    wp_tablet_pad_group.done event. This event is only sent when more than more
    than one mode is available.

    :param modes:
        the number of modes
    :type modes: `uint`
    """
    self._post_event(3, modes)


@ZwpTabletPadGroupV2.event("", [])
def done(self):
    """Tablet group description events sequence complete

    This event is sent immediately to signal the end of the initial burst of
    descriptive events. A client may consider the static description of the
    tablet to be complete and finalize initialization of the tablet group.
    """
    self._post_event(4)


@ZwpTabletPadGroupV2.event("uuu", [None, None, None])
def mode_switch(self, time, serial, mode):
    """Mode switch event

    Notification that the mode was switched.

    A mode applies to all buttons, rings and strips in a group simultaneously,
    but a client is not required to assign different actions for each mode. For
    example, a client may have mode-specific button mappings but map the ring
    to vertical scrolling in all modes. Mode indices start at 0.

    Switching modes is compositor-dependent. The compositor may provide visual
    cues to the client about the mode, e.g. by toggling LEDs on the tablet
    device. Mode-switching may be software-controlled or controlled by one or
    more physical buttons. For example, on a Wacom Intuos Pro, the button
    inside the ring may be assigned to switch between modes.

    The compositor will also send this event after wp_tablet_pad.enter on each
    group in order to notify of the current mode. Groups that only feature one
    mode will use mode=0 when emitting this event.

    If a button action in the new mode differs from the action in the previous
    mode, the client should immediately issue a wp_tablet_pad.set_feedback
    request for each changed button.

    If a ring or strip action in the new mode differs from the action in the
    previous mode, the client should immediately issue a
    wp_tablet_ring.set_feedback or wp_tablet_strip.set_feedback request for
    each changed ring or strip.

    :param time:
        the time of the event with millisecond granularity
    :type time: `uint`
    :param serial:
    :type serial: `uint`
    :param mode:
        the new mode of the pad
    :type mode: `uint`
    """
    self._post_event(5, time, serial, mode)


ZwpTabletPadGroupV2._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.interface import Interface
from .wl_requests import WlRequests

import enum


class WlCore(Interface):
    """Interface object

    The interface object with the most basic content.
    """
    name = "wl_core"
    version = 1

    the_enum = enum.Enum("the_enum", {
        "zero": 0,
        "one": 1,
        "hex_two": 0x2,
    })


@WlCore.request("niuf", [WlCore, None, None, None])
def make_request(self, the_int, the_uint, the_fixed):
    """A request

    The request asks the server for an event.

    :param the_int:
        the arg summary
    :type the_int: `int`
    :param the_uint:
    :type the_uint: `uint`
    :param the_fixed:
    :type the_fixed: `fixed`
    :returns:
        :class:`WlCore`
    """
    id = self._marshal_constructor(0, WlCore, the_int, the_uint, the_fixed)
    return id


@WlCore.request("iufn", [None, None, None, WlCore])
def make_request2(self, the_int, the_uint, the_fixed):
    """A request

    The request asks the server for an event but move the args around.

    :param the_int:
        the arg summary
    :type the_int: `int`
    :param the_uint:
    :type the_uint: `uint`
    :param the_fixed:
    :type the_fixed: `fixed`
    :returns:
        :class:`WlCore` -- a :class:`WlCore` object
    """
    id = self._marshal_constructor(1, WlCore, the_int, the_uint, the_fixed)
    return id


@WlCore.event("no", [WlCore, WlRequests])
def send_event(self, id, object):
    """A :class:`WlCore` event

    Send an event, but also put in some docs for our interface :class:`WlCore`,
    some other interface wl_other, a local function call :func:`WlCore.func()`,
    and another function call wl_other.func.

    :param id:
        a :class:`WlCore` object
    :type id: :class:`WlCore`
    :param object:
        a :class:`~pywayland.protocol.scanner_test.WlRequests` object
    :type object: :class:`~pywayland.protocol.scanner_test.WlRequests`
    """
    self._post_event(0, id, object)


WlCore._gen_c()



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .element import Element, Attribute, Child

from .description import Description
from .enum import Enum
from .event import Event
from .request import Request

import itertools


class Interface(Element):
    attributes = [Attribute('name', True), Attribute('version', True)]
    children = [
        Child('description', Description, False, False),
        Child('enum', Enum, False, True),
        # Child('request', Request, False, True),
        # Child('event', Event, False, True)
    ]

    def __init__(self, iface):
        """Scanner for interface objects

        Required attributes: `name` and `version`

        Child elements: `description`, `request`, `event`, `enum`
        """
        super(Interface, self).__init__(iface)

        # Requests and events need special handling to get the opcode and
        # interface name
        self.event = [
            Event(event, self.name, i)
            for i, event in enumerate(iface.findall('event'))
        ]
        self.request = [
            Request(request, self.name, i)
            for i, request in enumerate(iface.findall('request'))
        ]

    @property
    def class_name(self):
        """Returns the name of the class of the interface

        Camel cases the name of the interface, to be used as the class name.
        """
        return ''.join(x.capitalize() for x in self.name.split('_'))

    def output(self, printer, module_imports):
        """Generate the output for the interface to the printer"""
        # Imports
        imports = set(
            _import
            for method in itertools.chain(self.request, self.event)
            for _import in method.imports(module_imports)
        )
        printer('from pywayland.interface import Interface')
        for module, import_ in sorted(imports):
            printer('from {} import {}'.format(module, import_))
        if self.enum:
            printer()
            printer('import enum')
        printer()
        printer()

        # Class definition
        printer('class {}(Interface):'.format(self.class_name))
        with printer.indented():
            # Docstring
            if self.description:
                self.description.output(printer)
                printer('"""')

            # Class attributes
            printer('name = "{}"'.format(self.name))
            printer('version = {}'.format(self.version))

            # Enums
            for enum in self.enum:
                printer()
                enum.output(printer)

        # Events and requests
        for method in itertools.chain(self.request, self.event):
            printer()
            printer()
            method.output(printer, self.class_name, module_imports)

        printer()
        printer()
        printer('{}._gen_c()'.format(self.class_name))



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2015-2016 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlSurface
from .zxdg_exported_v2 import ZxdgExportedV2


class ZxdgExporterV2(Interface):
    """Interface for exporting surfaces

    A global interface used for exporting surfaces that can later be imported
    using xdg_importer.
    """
    name = "zxdg_exporter_v2"
    version = 1


@ZxdgExporterV2.request("", [])
def destroy(self):
    """Destroy the xdg_exporter object

    Notify the compositor that the xdg_exporter object will no longer be used.
    """
    self._marshal(0)
    self._destroy()


@ZxdgExporterV2.request("no", [ZxdgExportedV2, WlSurface])
def export_toplevel(self, surface):
    """Export a toplevel surface

    The export_toplevel request exports the passed surface so that it can later
    be imported via xdg_importer. When called, a new xdg_exported object will
    be created and xdg_exported.handle will be sent immediately. See the
    corresponding interface and event for details.

    A surface may be exported multiple times, and each exported handle may be
    used to create a xdg_imported multiple times. Only
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` equivalent surfaces may
    be exported.

    :param surface:
        the surface to export
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    :returns:
        :class:`~pywayland.protocol.xdg_foreign_unstable_v2.ZxdgExportedV2` --
        the new xdg_exported object
    """
    id = self._marshal_constructor(1, ZxdgExportedV2, surface)
    return id


ZxdgExporterV2._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .wl_region import WlRegion
from .wl_surface import WlSurface


class WlCompositor(Interface):
    """The compositor singleton

    A compositor.  This object is a singleton global.  The compositor is in
    charge of combining the contents of multiple surfaces into one displayable
    output.
    """
    name = "wl_compositor"
    version = 4


@WlCompositor.request("n", [WlSurface])
def create_surface(self):
    """Create new surface

    Ask the compositor to create a new surface.

    :returns:
        :class:`~pywayland.protocol.wayland.WlSurface` -- the new surface
    """
    id = self._marshal_constructor(0, WlSurface)
    return id


@WlCompositor.request("n", [WlRegion])
def create_region(self):
    """Create new region

    Ask the compositor to create a new region.

    :returns:
        :class:`~pywayland.protocol.wayland.WlRegion` -- the new region
    """
    id = self._marshal_constructor(1, WlRegion)
    return id


WlCompositor._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2012, 2013 Intel Corporation
# Copyright © 2015, 2016 Jan Arne Petersen
# Copyright © 2017, 2018 Red Hat, Inc.
# Copyright © 2018       Purism SPC
#
# Permission to use, copy, modify, distribute, and sell this
# software and its documentation for any purpose is hereby granted
# without fee, provided that the above copyright notice appear in
# all copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of
# the copyright holders not be used in advertising or publicity
# pertaining to distribution of the software without specific,
# written prior permission.  The copyright holders make no
# representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied
# warranty.
#
# THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
# SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
# SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
# THIS SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlSeat
from .zwp_text_input_v3 import ZwpTextInputV3


class ZwpTextInputManagerV3(Interface):
    """Text input manager

    A factory for text-input objects. This object is a global singleton.
    """
    name = "zwp_text_input_manager_v3"
    version = 1


@ZwpTextInputManagerV3.request("", [])
def destroy(self):
    """Destroy the wp_text_input_manager

    Destroy the wp_text_input_manager object.
    """
    self._marshal(0)
    self._destroy()


@ZwpTextInputManagerV3.request("no", [ZwpTextInputV3, WlSeat])
def get_text_input(self, seat):
    """Create a new text input object

    Creates a new text-input object for a given seat.

    :param seat:
    :type seat: :class:`~pywayland.protocol.wayland.WlSeat`
    :returns:
        :class:`~pywayland.protocol.text_input_unstable_v3.ZwpTextInputV3`
    """
    id = self._marshal_constructor(1, ZwpTextInputV3, seat)
    return id


ZwpTextInputManagerV3._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .wl_surface import WlSurface


class WlTouch(Interface):
    """Touchscreen input device

    The :class:`WlTouch` interface represents a touchscreen associated with a
    seat.

    Touch interactions can consist of one or more contacts. For each contact, a
    series of events is generated, starting with a down event, followed by zero
    or more motion events, and ending with an up event. Events relating to the
    same contact point can be identified by the ID of the sequence.
    """
    name = "wl_touch"
    version = 7


@WlTouch.request("3", [])
def release(self):
    """Release the touch object
    """
    self._marshal(0)
    self._destroy()


@WlTouch.event("uuoiff", [None, None, WlSurface, None, None, None])
def down(self, serial, time, surface, id, x, y):
    """Touch down event and beginning of a touch sequence

    A new touch point has appeared on the surface. This touch point is assigned
    a unique ID. Future events from this touch point reference this ID. The ID
    ceases to be valid after a touch up event and may be reused in the future.

    :param serial:
        serial number of the touch down event
    :type serial: `uint`
    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    :param surface:
        surface touched
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    :param id:
        the unique ID of this touch point
    :type id: `int`
    :param x:
        surface-local x coordinate
    :type x: `fixed`
    :param y:
        surface-local y coordinate
    :type y: `fixed`
    """
    self._post_event(0, serial, time, surface, id, x, y)


@WlTouch.event("uui", [None, None, None])
def up(self, serial, time, id):
    """End of a touch event sequence

    The touch point has disappeared. No further events will be sent for this
    touch point and the touch point's ID is released and may be reused in a
    future touch down event.

    :param serial:
        serial number of the touch up event
    :type serial: `uint`
    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    :param id:
        the unique ID of this touch point
    :type id: `int`
    """
    self._post_event(1, serial, time, id)


@WlTouch.event("uiff", [None, None, None, None])
def motion(self, time, id, x, y):
    """Update of touch point coordinates

    A touch point has changed coordinates.

    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    :param id:
        the unique ID of this touch point
    :type id: `int`
    :param x:
        surface-local x coordinate
    :type x: `fixed`
    :param y:
        surface-local y coordinate
    :type y: `fixed`
    """
    self._post_event(2, time, id, x, y)


@WlTouch.event("", [])
def frame(self):
    """End of touch frame event

    Indicates the end of a set of events that logically belong together. A
    client is expected to accumulate the data in all events within the frame
    before proceeding.

    A :func:`WlTouch.frame()` terminates at least one event but otherwise no
    guarantee is provided about the set of events within a frame. A client must
    assume that any state not updated in a frame is unchanged from the
    previously known state.
    """
    self._post_event(3)


@WlTouch.event("", [])
def cancel(self):
    """Touch session cancelled

    Sent if the compositor decides the touch stream is a global gesture. No
    further events are sent to the clients from that particular gesture. Touch
    cancellation applies to all touch points currently active on this client's
    surface. The client is responsible for finalizing the touch points, future
    touch points on this surface may reuse the touch point ID.
    """
    self._post_event(4)


@WlTouch.event("6iff", [None, None, None])
def shape(self, id, major, minor):
    """Update shape of touch point

    Sent when a touchpoint has changed its shape.

    This event does not occur on its own. It is sent before a
    :func:`WlTouch.frame()` event and carries the new shape information for any
    previously reported, or new touch points of that frame.

    Other events describing the touch point such as :func:`WlTouch.down()`,
    :func:`WlTouch.motion()` or :func:`WlTouch.orientation()` may be sent
    within the same :func:`WlTouch.frame()`. A client should treat these events
    as a single logical touch point update. The order of
    :func:`WlTouch.shape()`, :func:`WlTouch.orientation()` and
    :func:`WlTouch.motion()` is not guaranteed. A :func:`WlTouch.down()` event
    is guaranteed to occur before the first :func:`WlTouch.shape()` event for
    this touch ID but both events may occur within the same
    :func:`WlTouch.frame()`.

    A touchpoint shape is approximated by an ellipse through the major and
    minor axis length. The major axis length describes the longer diameter of
    the ellipse, while the minor axis length describes the shorter diameter.
    Major and minor are orthogonal and both are specified in surface-local
    coordinates. The center of the ellipse is always at the touchpoint location
    as reported by :func:`WlTouch.down()` or :func:`WlTouch.move()`.

    This event is only sent by the compositor if the touch device supports
    shape reports. The client has to make reasonable assumptions about the
    shape if it did not receive this event.

    :param id:
        the unique ID of this touch point
    :type id: `int`
    :param major:
        length of the major axis in surface-local coordinates
    :type major: `fixed`
    :param minor:
        length of the minor axis in surface-local coordinates
    :type minor: `fixed`
    """
    self._post_event(5, id, major, minor)


@WlTouch.event("6if", [None, None])
def orientation(self, id, orientation):
    """Update orientation of touch point

    Sent when a touchpoint has changed its orientation.

    This event does not occur on its own. It is sent before a
    :func:`WlTouch.frame()` event and carries the new shape information for any
    previously reported, or new touch points of that frame.

    Other events describing the touch point such as :func:`WlTouch.down()`,
    :func:`WlTouch.motion()` or :func:`WlTouch.shape()` may be sent within the
    same :func:`WlTouch.frame()`. A client should treat these events as a
    single logical touch point update. The order of :func:`WlTouch.shape()`,
    :func:`WlTouch.orientation()` and :func:`WlTouch.motion()` is not
    guaranteed. A :func:`WlTouch.down()` event is guaranteed to occur before
    the first :func:`WlTouch.orientation()` event for this touch ID but both
    events may occur within the same :func:`WlTouch.frame()`.

    The orientation describes the clockwise angle of a touchpoint's major axis
    to the positive surface y-axis and is normalized to the -180 to +180 degree
    range. The granularity of orientation depends on the touch device, some
    devices only support binary rotation values between 0 and 90 degrees.

    This event is only sent by the compositor if the touch device supports
    orientation reports.

    :param id:
        the unique ID of this touch point
    :type id: `int`
    :param orientation:
        angle between major axis and positive surface y-axis in degrees
    :type orientation: `fixed`
    """
    self._post_event(6, id, orientation)


WlTouch._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .wl_buffer import WlBuffer


class WlShmPool(Interface):
    """A shared memory pool

    The :class:`WlShmPool` object encapsulates a piece of memory shared between
    the compositor and client.  Through the :class:`WlShmPool` object, the
    client can allocate shared memory
    :class:`~pywayland.protocol.wayland.WlBuffer` objects. All objects created
    through the same pool share the same underlying mapped memory. Reusing the
    mapped memory avoids the setup/teardown overhead and is useful when
    interactively resizing a surface or for many small buffers.
    """
    name = "wl_shm_pool"
    version = 1


@WlShmPool.request("niiiiu", [WlBuffer, None, None, None, None, None])
def create_buffer(self, offset, width, height, stride, format):
    """Create a buffer from the pool

    Create a :class:`~pywayland.protocol.wayland.WlBuffer` object from the
    pool.

    The buffer is created offset bytes into the pool and has width and height
    as specified.  The stride argument specifies the number of bytes from the
    beginning of one row to the beginning of the next.  The format is the pixel
    format of the buffer and must be one of those advertised through the
    :func:`WlShm.format() <pywayland.protocol.wayland.WlShm..format>` event.

    A buffer will keep a reference to the pool it was created from so it is
    valid to destroy the pool immediately after creating a buffer from it.

    :param offset:
        buffer byte offset within the pool
    :type offset: `int`
    :param width:
        buffer width, in pixels
    :type width: `int`
    :param height:
        buffer height, in pixels
    :type height: `int`
    :param stride:
        number of bytes from the beginning of one row to the beginning of the
        next row
    :type stride: `int`
    :param format:
        buffer pixel format
    :type format: `uint`
    :returns:
        :class:`~pywayland.protocol.wayland.WlBuffer` -- buffer to create
    """
    id = self._marshal_constructor(0, WlBuffer, offset, width, height, stride, format)
    return id


@WlShmPool.request("", [])
def destroy(self):
    """Destroy the pool

    Destroy the shared memory pool.

    The mmapped memory will be released when all buffers that have been created
    from this pool are gone.
    """
    self._marshal(1)
    self._destroy()


@WlShmPool.request("i", [None])
def resize(self, size):
    """Change the size of the pool mapping

    This request will cause the server to remap the backing memory for the pool
    from the file descriptor passed when the pool was created, but using the
    new size.  This request can only be used to make the pool bigger.

    :param size:
        new size of the pool, in bytes
    :type size: `int`
    """
    self._marshal(2, size)


WlShmPool._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlOutput
from ..wayland import WlSeat

import enum


class ZxdgToplevelV6(Interface):
    """Toplevel surface

    This interface defines an :class:`~pywayland.protocol.xdg_shell.XdgSurface`
    role which allows a surface to, among other things, set window-like
    properties such as maximize, fullscreen, and minimize, set application-
    specific metadata like title and id, and well as trigger user interactive
    operations such as interactive resize and move.
    """
    name = "zxdg_toplevel_v6"
    version = 1

    resize_edge = enum.Enum("resize_edge", {
        "none": 0,
        "top": 1,
        "bottom": 2,
        "left": 4,
        "top_left": 5,
        "bottom_left": 6,
        "right": 8,
        "top_right": 9,
        "bottom_right": 10,
    })

    state = enum.Enum("state", {
        "maximized": 1,
        "fullscreen": 2,
        "resizing": 3,
        "activated": 4,
    })


@ZxdgToplevelV6.request("", [])
def destroy(self):
    """Destroy the :class:`~pywayland.protocol.xdg_shell.XdgToplevel`

    Unmap and destroy the window. The window will be effectively hidden from
    the user's point of view, and all state like maximization, fullscreen, and
    so on, will be lost.
    """
    self._marshal(0)
    self._destroy()


@ZxdgToplevelV6.request("?o", [ZxdgToplevelV6])
def set_parent(self, parent):
    """Set the parent of this surface

    Set the "parent" of this surface. This window should be stacked above a
    parent. The parent surface must be mapped as long as this surface is
    mapped.

    Parent windows should be set on dialogs, toolboxes, or other "auxiliary"
    surfaces, so that the parent is raised when the dialog is raised.

    :param parent:
    :type parent: :class:`ZxdgToplevelV6` or `None`
    """
    self._marshal(1, parent)


@ZxdgToplevelV6.request("s", [None])
def set_title(self, title):
    """Set surface title

    Set a short title for the surface.

    This string may be used to identify the surface in a task bar, window list,
    or other user interface elements provided by the compositor.

    The string must be encoded in UTF-8.

    :param title:
    :type title: `string`
    """
    self._marshal(2, title)


@ZxdgToplevelV6.request("s", [None])
def set_app_id(self, app_id):
    """Set application id

    Set an application identifier for the surface.

    The app ID identifies the general class of applications to which the
    surface belongs. The compositor can use this to group multiple surfaces
    together, or to determine how to launch a new application.

    For D-Bus activatable applications, the app ID is used as the D-Bus service
    name.

    The compositor shell will try to group application surfaces together by
    their app ID. As a best practice, it is suggested to select app ID's that
    match the basename of the application's .desktop file. For example,
    "org.freedesktop.FooViewer" where the .desktop file is
    "org.freedesktop.FooViewer.desktop".

    See the desktop-entry specification [0] for more details on application
    identifiers and how they relate to well-known D-Bus names and .desktop
    files.

    [0] http://standards.freedesktop.org/desktop-entry-spec/

    :param app_id:
    :type app_id: `string`
    """
    self._marshal(3, app_id)


@ZxdgToplevelV6.request("ouii", [WlSeat, None, None, None])
def show_window_menu(self, seat, serial, x, y):
    """Show the window menu

    Clients implementing client-side decorations might want to show a context
    menu when right-clicking on the decorations, giving the user a menu that
    they can use to maximize or minimize the window.

    This request asks the compositor to pop up such a window menu at the given
    position, relative to the local surface coordinates of the parent surface.
    There are no guarantees as to what menu items the window menu contains.

    This request must be used in response to some sort of user action like a
    button press, key press, or touch down event.

    :param seat:
        the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
    :type seat: :class:`~pywayland.protocol.wayland.WlSeat`
    :param serial:
        the serial of the user event
    :type serial: `uint`
    :param x:
        the x position to pop up the window menu at
    :type x: `int`
    :param y:
        the y position to pop up the window menu at
    :type y: `int`
    """
    self._marshal(4, seat, serial, x, y)


@ZxdgToplevelV6.request("ou", [WlSeat, None])
def move(self, seat, serial):
    """Start an interactive move

    Start an interactive, user-driven move of the surface.

    This request must be used in response to some sort of user action like a
    button press, key press, or touch down event. The passed serial is used to
    determine the type of interactive move (touch, pointer, etc).

    The server may ignore move requests depending on the state of the surface
    (e.g. fullscreen or maximized), or if the passed serial is no longer valid.

    If triggered, the surface will lose the focus of the device
    (:class:`~pywayland.protocol.wayland.WlPointer`,
    :class:`~pywayland.protocol.wayland.WlTouch`, etc) used for the move. It is
    up to the compositor to visually indicate that the move is taking place,
    such as updating a pointer cursor, during the move. There is no guarantee
    that the device focus will return when the move is completed.

    :param seat:
        the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
    :type seat: :class:`~pywayland.protocol.wayland.WlSeat`
    :param serial:
        the serial of the user event
    :type serial: `uint`
    """
    self._marshal(5, seat, serial)


@ZxdgToplevelV6.request("ouu", [WlSeat, None, None])
def resize(self, seat, serial, edges):
    """Start an interactive resize

    Start a user-driven, interactive resize of the surface.

    This request must be used in response to some sort of user action like a
    button press, key press, or touch down event. The passed serial is used to
    determine the type of interactive resize (touch, pointer, etc).

    The server may ignore resize requests depending on the state of the surface
    (e.g. fullscreen or maximized).

    If triggered, the client will receive configure events with the "resize"
    state enum value and the expected sizes. See the "resize" enum value for
    more details about what is required. The client must also acknowledge
    configure events using "ack_configure". After the resize is completed, the
    client will receive another "configure" event without the resize state.

    If triggered, the surface also will lose the focus of the device
    (:class:`~pywayland.protocol.wayland.WlPointer`,
    :class:`~pywayland.protocol.wayland.WlTouch`, etc) used for the resize. It
    is up to the compositor to visually indicate that the resize is taking
    place, such as updating a pointer cursor, during the resize. There is no
    guarantee that the device focus will return when the resize is completed.

    The edges parameter specifies how the surface should be resized, and is one
    of the values of the resize_edge enum. The compositor may use this
    information to update the surface position for example when dragging the
    top left corner. The compositor may also use this information to adapt its
    behavior, e.g. choose an appropriate cursor image.

    :param seat:
        the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
    :type seat: :class:`~pywayland.protocol.wayland.WlSeat`
    :param serial:
        the serial of the user event
    :type serial: `uint`
    :param edges:
        which edge or corner is being dragged
    :type edges: `uint`
    """
    self._marshal(6, seat, serial, edges)


@ZxdgToplevelV6.request("ii", [None, None])
def set_max_size(self, width, height):
    """Set the maximum size

    Set a maximum size for the window.

    The client can specify a maximum size so that the compositor does not try
    to configure the window beyond this size.

    The width and height arguments are in window geometry coordinates. See
    :func:`XdgSurface.set_window_geometry()
    <pywayland.protocol.xdg_shell.XdgSurface..set_window_geometry>`.

    Values set in this way are double-buffered. They will get applied on the
    next commit.

    The compositor can use this information to allow or disallow different
    states like maximize or fullscreen and draw accurate animations.

    Similarly, a tiling window manager may use this information to place and
    resize client windows in a more effective way.

    The client should not rely on the compositor to obey the maximum size. The
    compositor may decide to ignore the values set by the client and request a
    larger size.

    If never set, or a value of zero in the request, means that the client has
    no expected maximum size in the given dimension. As a result, a client
    wishing to reset the maximum size to an unspecified state can use zero for
    width and height in the request.

    Requesting a maximum size to be smaller than the minimum size of a surface
    is illegal and will result in a protocol error.

    The width and height must be greater than or equal to zero. Using strictly
    negative values for width and height will result in a protocol error.

    :param width:
    :type width: `int`
    :param height:
    :type height: `int`
    """
    self._marshal(7, width, height)


@ZxdgToplevelV6.request("ii", [None, None])
def set_min_size(self, width, height):
    """Set the minimum size

    Set a minimum size for the window.

    The client can specify a minimum size so that the compositor does not try
    to configure the window below this size.

    The width and height arguments are in window geometry coordinates. See
    :func:`XdgSurface.set_window_geometry()
    <pywayland.protocol.xdg_shell.XdgSurface..set_window_geometry>`.

    Values set in this way are double-buffered. They will get applied on the
    next commit.

    The compositor can use this information to allow or disallow different
    states like maximize or fullscreen and draw accurate animations.

    Similarly, a tiling window manager may use this information to place and
    resize client windows in a more effective way.

    The client should not rely on the compositor to obey the minimum size. The
    compositor may decide to ignore the values set by the client and request a
    smaller size.

    If never set, or a value of zero in the request, means that the client has
    no expected minimum size in the given dimension. As a result, a client
    wishing to reset the minimum size to an unspecified state can use zero for
    width and height in the request.

    Requesting a minimum size to be larger than the maximum size of a surface
    is illegal and will result in a protocol error.

    The width and height must be greater than or equal to zero. Using strictly
    negative values for width and height will result in a protocol error.

    :param width:
    :type width: `int`
    :param height:
    :type height: `int`
    """
    self._marshal(8, width, height)


@ZxdgToplevelV6.request("", [])
def set_maximized(self):
    """Maximize the window

    Maximize the surface.

    After requesting that the surface should be maximized, the compositor will
    respond by emitting a configure event with the "maximized" state and the
    required window geometry. The client should then update its content,
    drawing it in a maximized state, i.e. without shadow or other decoration
    outside of the window geometry. The client must also acknowledge the
    configure when committing the new content (see ack_configure).

    It is up to the compositor to decide how and where to maximize the surface,
    for example which output and what region of the screen should be used.

    If the surface was already maximized, the compositor will still emit a
    configure event with the "maximized" state.
    """
    self._marshal(9)


@ZxdgToplevelV6.request("", [])
def unset_maximized(self):
    """Unmaximize the window

    Unmaximize the surface.

    After requesting that the surface should be unmaximized, the compositor
    will respond by emitting a configure event without the "maximized" state.
    If available, the compositor will include the window geometry dimensions
    the window had prior to being maximized in the configure request. The
    client must then update its content, drawing it in a regular state, i.e.
    potentially with shadow, etc. The client must also acknowledge the
    configure when committing the new content (see ack_configure).

    It is up to the compositor to position the surface after it was
    unmaximized; usually the position the surface had before maximizing, if
    applicable.

    If the surface was already not maximized, the compositor will still emit a
    configure event without the "maximized" state.
    """
    self._marshal(10)


@ZxdgToplevelV6.request("?o", [WlOutput])
def set_fullscreen(self, output):
    """Set the window as fullscreen on a monitor

    Make the surface fullscreen.

    You can specify an output that you would prefer to be fullscreen. If this
    value is NULL, it's up to the compositor to choose which display will be
    used to map this surface.

    If the surface doesn't cover the whole output, the compositor will position
    the surface in the center of the output and compensate with black borders
    filling the rest of the output.

    :param output:
    :type output: :class:`~pywayland.protocol.wayland.WlOutput` or `None`
    """
    self._marshal(11, output)


@ZxdgToplevelV6.request("", [])
def unset_fullscreen(self):
    """unset_fullscreen
    """
    self._marshal(12)


@ZxdgToplevelV6.request("", [])
def set_minimized(self):
    """Set the window as minimized

    Request that the compositor minimize your surface. There is no way to know
    if the surface is currently minimized, nor is there any way to unset
    minimization on this surface.

    If you are looking to throttle redrawing when minimized, please instead use
    the :func:`WlSurface.frame() <pywayland.protocol.wayland.WlSurface..frame>`
    event for this, as this will also work with live previews on windows in
    Alt-Tab, Expose or similar compositor features.
    """
    self._marshal(13)


@ZxdgToplevelV6.event("iia", [None, None, None])
def configure(self, width, height, states):
    """Suggest a surface change

    This configure event asks the client to resize its toplevel surface or to
    change its state. The configured state should not be applied immediately.
    See :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface..configure>` for details.

    The width and height arguments specify a hint to the window about how its
    surface should be resized in window geometry coordinates. See
    set_window_geometry.

    If the width or height arguments are zero, it means the client should
    decide its own window dimension. This may happen when the compositor needs
    to configure the state of the surface but doesn't have any information
    about any previous or expected dimension.

    The states listed in the event specify how the width/height arguments
    should be interpreted, and possibly how it should be drawn.

    Clients must send an ack_configure in response to this event. See
    :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface..configure>` and
    :func:`XdgSurface.ack_configure()
    <pywayland.protocol.xdg_shell.XdgSurface..ack_configure>` for details.

    :param width:
    :type width: `int`
    :param height:
    :type height: `int`
    :param states:
    :type states: `array`
    """
    self._post_event(0, width, height, states)


@ZxdgToplevelV6.event("", [])
def close(self):
    """Surface wants to be closed

    The close event is sent by the compositor when the user wants the surface
    to be closed. This should be equivalent to the user clicking the close
    button in client-side decorations, if your application has any.

    This is only a request that the user intends to close the window. The
    client may choose to ignore this request, or show a dialog to ask the user
    to save their data, etc.
    """
    self._post_event(1)


ZxdgToplevelV6._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .zwp_tablet_tool_v1 import ZwpTabletToolV1
from .zwp_tablet_v1 import ZwpTabletV1


class ZwpTabletSeatV1(Interface):
    """Controller object for graphic tablet devices of a seat

    An object that provides access to the graphics tablets available on this
    seat. After binding to this interface, the compositor sends a set of
    wp_tablet_seat.tablet_added and wp_tablet_seat.tool_added events.
    """
    name = "zwp_tablet_seat_v1"
    version = 1


@ZwpTabletSeatV1.request("", [])
def destroy(self):
    """Release the memory for the tablet seat object

    Destroy the wp_tablet_seat object. Objects created from this object are
    unaffected and should be destroyed separately.
    """
    self._marshal(0)
    self._destroy()


@ZwpTabletSeatV1.event("n", [ZwpTabletV1])
def tablet_added(self, id):
    """New device notification

    This event is sent whenever a new tablet becomes available on this seat.
    This event only provides the object id of the tablet, any static
    information about the tablet (device name, vid/pid, etc.) is sent through
    the wp_tablet interface.

    :param id:
        the newly added graphics tablet
    :type id: :class:`~pywayland.protocol.tablet_unstable_v1.ZwpTabletV1`
    """
    self._post_event(0, id)


@ZwpTabletSeatV1.event("n", [ZwpTabletToolV1])
def tool_added(self, id):
    """A new tool has been used with a tablet

    This event is sent whenever a tool that has not previously been used with a
    tablet comes into use. This event only provides the object id of the tool;
    any static information about the tool (capabilities, type, etc.) is sent
    through the wp_tablet_tool interface.

    :param id:
        the newly added tablet tool
    :type id: :class:`~pywayland.protocol.tablet_unstable_v1.ZwpTabletToolV1`
    """
    self._post_event(1, id)


ZwpTabletSeatV1._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2015-2016 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from .zxdg_imported_v2 import ZxdgImportedV2


class ZxdgImporterV2(Interface):
    """Interface for importing surfaces

    A global interface used for importing surfaces exported by xdg_exporter.
    With this interface, a client can create a reference to a surface of
    another client.
    """
    name = "zxdg_importer_v2"
    version = 1


@ZxdgImporterV2.request("", [])
def destroy(self):
    """Destroy the xdg_importer object

    Notify the compositor that the xdg_importer object will no longer be used.
    """
    self._marshal(0)
    self._destroy()


@ZxdgImporterV2.request("ns", [ZxdgImportedV2, None])
def import_toplevel(self, handle):
    """Import a toplevel surface

    The import_toplevel request imports a surface from any client given a
    handle retrieved by exporting said surface using
    xdg_exporter.export_toplevel. When called, a new xdg_imported object will
    be created. This new object represents the imported surface, and the
    importing client can manipulate its relationship using it. See xdg_imported
    for details.

    :param handle:
        the exported surface handle
    :type handle: `string`
    :returns:
        :class:`~pywayland.protocol.xdg_foreign_unstable_v2.ZxdgImportedV2` --
        the new xdg_imported object
    """
    id = self._marshal_constructor(1, ZxdgImportedV2, handle)
    return id


ZxdgImporterV2._gen_c()



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from weakref import WeakKeyDictionary, WeakValueDictionary
from typing import List  # noqa: F401

from pywayland import ffi
from .dispatcher import Dispatcher
from .message import Message

weakkeydict = WeakKeyDictionary()  # type: WeakKeyDictionary


class classproperty:
    def __init__(self, f):
        self.f = f

    def __get__(self, obj, owner):
        return self.f(owner)


class InterfaceMeta(type):
    """Metaclass for Interfaces

    Initializes empty lists for events and requests for the given class.
    """
    def __init__(self, name, bases, dct):
        self.events = []
        self.requests = []

        # Initialize the interface cdata
        self._ptr = ffi.new("struct wl_interface *")


class Interface(metaclass=InterfaceMeta):
    """Wrapper class for wl_wayland structs

    Base class for interfaces that are defined by the wayland.xml class and
    generated by the scanner.  Sub-classes should use the
    :class:`InterfaceMeta` metaclass, which will define subclass.events and
    subclass.requests, the lists of the methods on this interface.  These class
    variables are populated by the :func:`Interface.event` and
    :func:`Interface.request` decorators.
    """

    _ptr = None
    name = None  # type: str
    version = None  # type: int

    @classproperty
    def proxy_class(interface):
        """Return a proxy class for the given interface

        :returns: :class:`~pywayland.client.proxy.Proxy` class for the given
                  interface
        """
        from pywayland.client.proxy import Proxy

        # Use the name of the interface to construct the class name
        class_name = '{}Proxy'.format(interface.__name__)
        # Extract the requests
        # TODO: add the enums to the class as well
        dct = {msg.name: msg.py_func for msg in interface.requests}
        # Construct a dispatcher
        dispacter_name = '{}Dispatcher'.format(interface.__name__)
        dispacter_class = type(dispacter_name, (Dispatcher,), {})
        # Add the interface and dispacter as a class attribute
        dct['_interface'] = interface
        dct['dispatcher'] = dispacter_class(interface.events)
        dct['registry'] = interface.registry

        # Return the new class
        return type(class_name, (Proxy,), dct)

    @classproperty
    def resource_class(interface):
        """Return a resource class for the given interface

        :returns: :class:`~pywayland.server.resource.Resource` class for the
                  given interface
        """
        from pywayland.server.resource import Resource

        # Use the name of the interface to construct the class name
        class_name = '{}Resource'.format(interface.__name__)
        # Extract the events
        dct = {msg.name: msg.py_func for msg in interface.events}
        # Construct a dispacter
        dispacter_name = '{}Dispatcher'.format(interface.__name__)
        dispacter_class = type(dispacter_name, (Dispatcher,), {})
        # Add the interface and dispacter as a class attribute
        dct['_interface'] = interface
        dct['dispatcher'] = dispacter_class(interface.requests, destructor=True)

        # Return the new class
        return type(class_name, (Resource,), dct)

    @classproperty
    def global_class(interface):
        """Return a global object for the given interface

        :returns: :class:`~pywayland.server.globals.Global` class for the given
                  interface
        """
        from pywayland.server.globals import Global

        # Use the name of the interface to construct the class name
        class_name = '{}Global'.format(interface.__name__)

        # Return the new class with the interface added as a class attribute
        return type(class_name, (Global,), {'_interface': interface})

    @classmethod
    def event(cls, signature, types):
        """Decorator for interface events

        Adds the decorated method to the list of events of the interface
        (server-side method).

        :param signature: Encodes the types of the arguments to the decorated
                          function.
        :type signature: `string`
        :param types: List of the types of any objects included in the argument
                      list, None if otherwise.
        :type types: `list`
        """
        def wrapper(func):
            cls.events.append(Message(func, signature, types))
            return func
        return wrapper

    @classmethod
    def request(cls, signature, types):
        """Decorator for interface requests

        Adds the decorated method to the list of requests of the interface
        (client-side method).

        :param signature: Encodes the types of the arguments to the decorated
                          function.
        :type signature: `string`
        :param types: List of the types of any objects included in the argument
                      list, None if otherwise.
        :type types: list
        """
        def wrapper(func):
            cls.requests.append(Message(func, signature, types))
            return func
        return wrapper

    @classmethod
    def _gen_c(cls):
        """Creates the wl_interface C struct

        Generates the CFFI cdata for the wl_interface struct given by the
        interface.
        """
        cls.registry = WeakValueDictionary()

        cls._ptr.name = name = ffi.new('char[]', cls.name.encode())
        cls._ptr.version = cls.version

        # Determine the number of methods to assign and assign them
        cls._ptr.method_count = len(cls.requests)
        cls._ptr.methods = methods_ptr = ffi.new("struct wl_message[]", len(cls.requests))
        # Iterate over the methods
        for i, message in enumerate(cls.requests):
            # First, generate the wl_message cdata
            msg_buf = ffi.buffer(message._ptr)
            methods_buf = ffi.buffer(methods_ptr + i)
            # Copy the contents of the cdata into the allocated cdata
            methods_buf[:] = msg_buf

        cls._ptr.event_count = len(cls.events)
        cls._ptr.events = events_ptr = ffi.new("struct wl_message[]", len(cls.events))
        # Iterate over the methods
        for i, message in enumerate(cls.events):
            # First, generate the wl_message cdata
            msg_buf = ffi.buffer(message._ptr)
            events_buf = ffi.buffer(events_ptr + i)
            # Copy the contents of the cdata into the allocated cdata
            events_buf[:] = msg_buf

        weakkeydict[cls._ptr] = (name, methods_ptr, events_ptr)



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2015, 2016 Red Hat
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface


class ZwpPrimarySelectionSourceV1(Interface):
    """Offer to replace the contents of the primary selection

    The source side of a wp_primary_selection_offer, it provides a way to
    describe the offered data and respond to requests to transfer the requested
    contents of the primary selection clipboard.
    """
    name = "zwp_primary_selection_source_v1"
    version = 1


@ZwpPrimarySelectionSourceV1.request("s", [None])
def offer(self, mime_type):
    """Add an offered mime type

    This request adds a mime type to the set of mime types advertised to
    targets. Can be called several times to offer multiple types.

    :param mime_type:
    :type mime_type: `string`
    """
    self._marshal(0, mime_type)


@ZwpPrimarySelectionSourceV1.request("", [])
def destroy(self):
    """Destroy the primary selection source

    Destroy the primary selection source.
    """
    self._marshal(1)
    self._destroy()


@ZwpPrimarySelectionSourceV1.event("sh", [None, None])
def send(self, mime_type, fd):
    """Send the primary selection contents

    Request for the current primary selection contents from the client. Send
    the specified mime type over the passed file descriptor, then close it.

    :param mime_type:
    :type mime_type: `string`
    :param fd:
    :type fd: `fd`
    """
    self._post_event(0, mime_type, fd)


@ZwpPrimarySelectionSourceV1.event("", [])
def cancelled(self):
    """Request for primary selection contents was canceled

    This primary selection source is no longer valid. The client should clean
    up and destroy this primary selection source.
    """
    self._post_event(1)


ZwpPrimarySelectionSourceV1._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface

import enum


class WlDataOffer(Interface):
    """Offer to transfer data

    A :class:`WlDataOffer` represents a piece of data offered for transfer by
    another client (the source client).  It is used by the copy-and-paste and
    drag-and-drop mechanisms.  The offer describes the different mime types
    that the data can be converted to and provides the mechanism for
    transferring the data directly from the source client.
    """
    name = "wl_data_offer"
    version = 3

    error = enum.Enum("error", {
        "invalid_finish": 0,
        "invalid_action_mask": 1,
        "invalid_action": 2,
        "invalid_offer": 3,
    })


@WlDataOffer.request("u?s", [None, None])
def accept(self, serial, mime_type):
    """Accept one of the offered mime types

    Indicate that the client can accept the given mime type, or NULL for not
    accepted.

    For objects of version 2 or older, this request is used by the client to
    give feedback whether the client can receive the given mime type, or NULL
    if none is accepted; the feedback does not determine whether the drag-and-
    drop operation succeeds or not.

    For objects of version 3 or newer, this request determines the final result
    of the drag-and-drop operation. If the end result is that no mime types
    were accepted, the drag-and-drop operation will be cancelled and the
    corresponding drag source will receive :func:`WlDataSource.cancelled()
    <pywayland.protocol.wayland.WlDataSource..cancelled>`. Clients may still
    use this event in conjunction with :func:`WlDataSource.action()
    <pywayland.protocol.wayland.WlDataSource..action>` for feedback.

    :param serial:
        serial number of the accept request
    :type serial: `uint`
    :param mime_type:
        mime type accepted by the client
    :type mime_type: `string` or `None`
    """
    self._marshal(0, serial, mime_type)


@WlDataOffer.request("sh", [None, None])
def receive(self, mime_type, fd):
    """Request that the data is transferred

    To transfer the offered data, the client issues this request and indicates
    the mime type it wants to receive.  The transfer happens through the passed
    file descriptor (typically created with the pipe system call).  The source
    client writes the data in the mime type representation requested and then
    closes the file descriptor.

    The receiving client reads from the read end of the pipe until EOF and then
    closes its end, at which point the transfer is complete.

    This request may happen multiple times for different mime types, both
    before and after :func:`WlDataDevice.drop()
    <pywayland.protocol.wayland.WlDataDevice..drop>`. Drag-and-drop destination
    clients may preemptively fetch data or examine it more closely to determine
    acceptance.

    :param mime_type:
        mime type desired by receiver
    :type mime_type: `string`
    :param fd:
        file descriptor for data transfer
    :type fd: `fd`
    """
    self._marshal(1, mime_type, fd)


@WlDataOffer.request("", [])
def destroy(self):
    """Destroy data offer

    Destroy the data offer.
    """
    self._marshal(2)
    self._destroy()


@WlDataOffer.request("3", [])
def finish(self):
    """The offer will no longer be used

    Notifies the compositor that the drag destination successfully finished the
    drag-and-drop operation.

    Upon receiving this request, the compositor will emit
    :func:`WlDataSource.dnd_finished()
    <pywayland.protocol.wayland.WlDataSource..dnd_finished>` on the drag source
    client.

    It is a client error to perform other requests than
    :func:`WlDataOffer.destroy()` after this one. It is also an error to
    perform this request after a NULL mime type has been set in
    :func:`WlDataOffer.accept()` or no action was received through
    :func:`WlDataOffer.action()`.
    """
    self._marshal(3)


@WlDataOffer.request("3uu", [None, None])
def set_actions(self, dnd_actions, preferred_action):
    """Set the available/preferred drag-and-drop actions

    Sets the actions that the destination side client supports for this
    operation. This request may trigger the emission of
    :func:`WlDataSource.action()
    <pywayland.protocol.wayland.WlDataSource..action>` and
    :func:`WlDataOffer.action()` events if the compositor needs to change the
    selected action.

    This request can be called multiple times throughout the drag-and-drop
    operation, typically in response to :func:`WlDataDevice.enter()
    <pywayland.protocol.wayland.WlDataDevice..enter>` or
    :func:`WlDataDevice.motion()
    <pywayland.protocol.wayland.WlDataDevice..motion>` events.

    This request determines the final result of the drag-and-drop operation. If
    the end result is that no action is accepted, the drag source will receive
    wl_drag_source.cancelled.

    The dnd_actions argument must contain only values expressed in the
    :func:`WlDataDeviceManager.dnd_actions()
    <pywayland.protocol.wayland.WlDataDeviceManager..dnd_actions>` enum, and
    the preferred_action argument must only contain one of those values set,
    otherwise it will result in a protocol error.

    While managing an "ask" action, the destination drag-and-drop client may
    perform further :func:`WlDataOffer.receive()` requests, and is expected to
    perform one last :func:`WlDataOffer.set_actions()` request with a preferred
    action other than "ask" (and optionally :func:`WlDataOffer.accept()`)
    before requesting :func:`WlDataOffer.finish()`, in order to convey the
    action selected by the user. If the preferred action is not in the
    :func:`WlDataOffer.source_actions()` mask, an error will be raised.

    If the "ask" action is dismissed (e.g. user cancellation), the client is
    expected to perform :func:`WlDataOffer.destroy()` right away.

    This request can only be made on drag-and-drop offers, a protocol error
    will be raised otherwise.

    :param dnd_actions:
        actions supported by the destination client
    :type dnd_actions: `uint`
    :param preferred_action:
        action preferred by the destination client
    :type preferred_action: `uint`
    """
    self._marshal(4, dnd_actions, preferred_action)


@WlDataOffer.event("s", [None])
def offer(self, mime_type):
    """Advertise offered mime type

    Sent immediately after creating the :class:`WlDataOffer` object.  One event
    per offered mime type.

    :param mime_type:
        offered mime type
    :type mime_type: `string`
    """
    self._post_event(0, mime_type)


@WlDataOffer.event("3u", [None])
def source_actions(self, source_actions):
    """Notify the source-side available actions

    This event indicates the actions offered by the data source. It will be
    sent right after :func:`WlDataDevice.enter()
    <pywayland.protocol.wayland.WlDataDevice..enter>`, or anytime the source
    side changes its offered actions through :func:`WlDataSource.set_actions()
    <pywayland.protocol.wayland.WlDataSource..set_actions>`.

    :param source_actions:
        actions offered by the data source
    :type source_actions: `uint`
    """
    self._post_event(1, source_actions)


@WlDataOffer.event("3u", [None])
def action(self, dnd_action):
    """Notify the selected action

    This event indicates the action selected by the compositor after matching
    the source/destination side actions. Only one action (or none) will be
    offered here.

    This event can be emitted multiple times during the drag-and-drop operation
    in response to destination side action changes through
    :func:`WlDataOffer.set_actions()`.

    This event will no longer be emitted after :func:`WlDataDevice.drop()
    <pywayland.protocol.wayland.WlDataDevice..drop>` happened on the drag-and-
    drop destination, the client must honor the last action received, or the
    last preferred one set through :func:`WlDataOffer.set_actions()` when
    handling an "ask" action.

    Compositors may also change the selected action on the fly, mainly in
    response to keyboard modifier changes during the drag-and-drop operation.

    The most recent action received is always the valid one. Prior to receiving
    :func:`WlDataDevice.drop()
    <pywayland.protocol.wayland.WlDataDevice..drop>`, the chosen action may
    change (e.g. due to keyboard modifiers being pressed). At the time of
    receiving :func:`WlDataDevice.drop()
    <pywayland.protocol.wayland.WlDataDevice..drop>` the drag-and-drop
    destination must honor the last action received.

    Action changes may still happen after :func:`WlDataDevice.drop()
    <pywayland.protocol.wayland.WlDataDevice..drop>`, especially on "ask"
    actions, where the drag-and-drop destination may choose another action
    afterwards. Action changes happening at this stage are always the result of
    inter-client negotiation, the compositor shall no longer be able to induce
    a different action.

    Upon "ask" actions, it is expected that the drag-and-drop destination may
    potentially choose a different action and/or mime type, based on
    :func:`WlDataOffer.source_actions()` and finally chosen by the user (e.g.
    popping up a menu with the available options). The final
    :func:`WlDataOffer.set_actions()` and :func:`WlDataOffer.accept()` requests
    must happen before the call to :func:`WlDataOffer.finish()`.

    :param dnd_action:
        action selected by the compositor
    :type dnd_action: `uint`
    """
    self._post_event(2, dnd_action)


WlDataOffer._gen_c()



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess
import sys

from setuptools import setup
from distutils.command.build import build
from setuptools.command.install import install
from setuptools.command.sdist import sdist

# we need this to import the version and scanner module directly so we can
# build the protocol before the cffi module has been compiled
sys.path.insert(0, 'pywayland')

from version import __version__ as pywayland_version

default_xml_file = '/usr/share/wayland/wayland.xml'


def get_protocol_command(klass):
    class ProtocolCommand(klass):
        user_options = [
            ('xml-file=', None, 'Location of wayland.xml protocol file'),
            ('output-dir=', None, 'Output location for protocol python files'),
            ('wayland-protocols', None, 'Force generation of external protocols from wayland-protocols'),
            ('no-wayland-protocols', None, 'Disable generation of external protocols from wayland-protocols')
        ] + klass.user_options
        boolean_options = ['wayland-protocols', 'no-wayland-protocols'] + klass.boolean_options

        def initialize_options(self):
            from scanner.__main__ import pkgconfig
            # try to figure out where the main wayland protocols are installed
            try:
                data_dir = pkgconfig('wayland-scanner', 'pkgdatadir')
            except subprocess.CalledProcessError:
                # silently fallback to the default
                self.xml_file = default_xml_file
            else:
                self.xml_file = os.path.join(data_dir, 'wayland.xml')

            self.output_dir = './pywayland/protocol'
            self.wayland_protocols = False
            self.no_wayland_protocols = False

            klass.initialize_options(self)

        def finalize_options(self):
            assert os.path.exists(self.xml_file), (
                "Specified Wayland protocol file, {}, does not exist "
                "please specify valid protocol file".format(self.xml_file)
            )

            klass.finalize_options(self)

        def run(self):
            from scanner import Protocol
            from scanner.__main__ import get_wayland_protocols

            # Generate the wayland interface by default
            input_files = [self.xml_file]
            modules = ["wayland"]

            # Unless users says don't build protocols, try to build them
            if not self.no_wayland_protocols:
                try:
                    protocol_modules, protocol_files = get_wayland_protocols()
                    input_files += protocol_files
                    modules += protocol_modules
                except Exception:
                    # but only complain if we ask specifically to build them
                    if self.wayland_protocols:
                        raise

            # Ensure the output dir exists
            if not os.path.isdir(self.output_dir):
                os.makedirs(self.output_dir, 0o775)

            # Run and scan all the above found xml files
            protocols = [Protocol(input_xml) for input_xml in input_files]
            protocol_imports = {
                interface.name: protocol.name
                for protocol in protocols
                for interface in protocol.interface
            }

            for protocol in protocols:
                protocol.output(self.output_dir, protocol_imports)

            # if we're building or installing, add the modules we generated
            if hasattr(self, 'distribution'):
                for module in modules:
                    self.distribution.packages.append('pywayland.protocol.{}'.format(module))

            klass.run(self)

    return ProtocolCommand


BuildCommand = get_protocol_command(build)
InstallCommand = get_protocol_command(install)
SdistCommand = get_protocol_command(sdist)

# For the purposes of uploading to PyPI, we'll get the version of Wayland here
try:
    from pywayland import __wayland_version__
    rst_input = open('README.rst').read().split('\n')
except Exception:
    long_description = ""
else:
    version = 'Built against Wayland {}\n'.format(__wayland_version__)
    rst_input.insert(3, version)

    long_description = '\n' + '\n'.join(rst_input)

setup(
    version=pywayland_version,
    long_description=long_description,
    long_description_content_type='text/x-rst',
    cmdclass={
        'build': BuildCommand,
        'install': InstallCommand,
        'sdist': SdistCommand
    },
    cffi_modules=['ffi_build.py:ffi_builder']
)



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .zwp_tablet_pad_v2 import ZwpTabletPadV2
from .zwp_tablet_tool_v2 import ZwpTabletToolV2
from .zwp_tablet_v2 import ZwpTabletV2


class ZwpTabletSeatV2(Interface):
    """Controller object for graphic tablet devices of a seat

    An object that provides access to the graphics tablets available on this
    seat. After binding to this interface, the compositor sends a set of
    wp_tablet_seat.tablet_added and wp_tablet_seat.tool_added events.
    """
    name = "zwp_tablet_seat_v2"
    version = 1


@ZwpTabletSeatV2.request("", [])
def destroy(self):
    """Release the memory for the tablet seat object

    Destroy the wp_tablet_seat object. Objects created from this object are
    unaffected and should be destroyed separately.
    """
    self._marshal(0)
    self._destroy()


@ZwpTabletSeatV2.event("n", [ZwpTabletV2])
def tablet_added(self, id):
    """New device notification

    This event is sent whenever a new tablet becomes available on this seat.
    This event only provides the object id of the tablet, any static
    information about the tablet (device name, vid/pid, etc.) is sent through
    the wp_tablet interface.

    :param id:
        the newly added graphics tablet
    :type id: :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletV2`
    """
    self._post_event(0, id)


@ZwpTabletSeatV2.event("n", [ZwpTabletToolV2])
def tool_added(self, id):
    """A new tool has been used with a tablet

    This event is sent whenever a tool that has not previously been used with a
    tablet comes into use. This event only provides the object id of the tool;
    any static information about the tool (capabilities, type, etc.) is sent
    through the wp_tablet_tool interface.

    :param id:
        the newly added tablet tool
    :type id: :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletToolV2`
    """
    self._post_event(1, id)


@ZwpTabletSeatV2.event("n", [ZwpTabletPadV2])
def pad_added(self, id):
    """New pad notification

    This event is sent whenever a new pad is known to the system. Typically,
    pads are physically attached to tablets and a pad_added event is sent
    immediately after the wp_tablet_seat.tablet_added. However, some standalone
    pad devices logically attach to tablets at runtime, and the client must
    wait for wp_tablet_pad.enter to know the tablet a pad is attached to.

    This event only provides the object id of the pad. All further features
    (buttons, strips, rings) are sent through the wp_tablet_pad interface.

    :param id:
        the newly added pad
    :type id: :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletPadV2`
    """
    self._post_event(2, id)


ZwpTabletSeatV2._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .wl_shm_pool import WlShmPool

import enum


class WlShm(Interface):
    """Shared memory support

    A singleton global object that provides support for shared memory.

    Clients can create :class:`~pywayland.protocol.wayland.WlShmPool` objects
    using the create_pool request.

    At connection setup time, the :class:`WlShm` object emits one or more
    format events to inform clients about the valid pixel formats that can be
    used for buffers.
    """
    name = "wl_shm"
    version = 1

    error = enum.Enum("error", {
        "invalid_format": 0,
        "invalid_stride": 1,
        "invalid_fd": 2,
    })

    format = enum.Enum("format", {
        "argb8888": 0,
        "xrgb8888": 1,
        "c8": 0x20203843,
        "rgb332": 0x38424752,
        "bgr233": 0x38524742,
        "xrgb4444": 0x32315258,
        "xbgr4444": 0x32314258,
        "rgbx4444": 0x32315852,
        "bgrx4444": 0x32315842,
        "argb4444": 0x32315241,
        "abgr4444": 0x32314241,
        "rgba4444": 0x32314152,
        "bgra4444": 0x32314142,
        "xrgb1555": 0x35315258,
        "xbgr1555": 0x35314258,
        "rgbx5551": 0x35315852,
        "bgrx5551": 0x35315842,
        "argb1555": 0x35315241,
        "abgr1555": 0x35314241,
        "rgba5551": 0x35314152,
        "bgra5551": 0x35314142,
        "rgb565": 0x36314752,
        "bgr565": 0x36314742,
        "rgb888": 0x34324752,
        "bgr888": 0x34324742,
        "xbgr8888": 0x34324258,
        "rgbx8888": 0x34325852,
        "bgrx8888": 0x34325842,
        "abgr8888": 0x34324241,
        "rgba8888": 0x34324152,
        "bgra8888": 0x34324142,
        "xrgb2101010": 0x30335258,
        "xbgr2101010": 0x30334258,
        "rgbx1010102": 0x30335852,
        "bgrx1010102": 0x30335842,
        "argb2101010": 0x30335241,
        "abgr2101010": 0x30334241,
        "rgba1010102": 0x30334152,
        "bgra1010102": 0x30334142,
        "yuyv": 0x56595559,
        "yvyu": 0x55595659,
        "uyvy": 0x59565955,
        "vyuy": 0x59555956,
        "ayuv": 0x56555941,
        "nv12": 0x3231564e,
        "nv21": 0x3132564e,
        "nv16": 0x3631564e,
        "nv61": 0x3136564e,
        "yuv410": 0x39565559,
        "yvu410": 0x39555659,
        "yuv411": 0x31315559,
        "yvu411": 0x31315659,
        "yuv420": 0x32315559,
        "yvu420": 0x32315659,
        "yuv422": 0x36315559,
        "yvu422": 0x36315659,
        "yuv444": 0x34325559,
        "yvu444": 0x34325659,
    })


@WlShm.request("nhi", [WlShmPool, None, None])
def create_pool(self, fd, size):
    """Create a shm pool

    Create a new :class:`~pywayland.protocol.wayland.WlShmPool` object.

    The pool can be used to create shared memory based buffer objects.  The
    server will mmap size bytes of the passed file descriptor, to use as
    backing memory for the pool.

    :param fd:
        file descriptor for the pool
    :type fd: `fd`
    :param size:
        pool size, in bytes
    :type size: `int`
    :returns:
        :class:`~pywayland.protocol.wayland.WlShmPool` -- pool to create
    """
    id = self._marshal_constructor(0, WlShmPool, fd, size)
    return id


@WlShm.event("u", [None])
def format(self, format):
    """Pixel format description

    Informs the client about a valid pixel format that can be used for buffers.
    Known formats include argb8888 and xrgb8888.

    :param format:
        buffer pixel format
    :type format: `uint`
    """
    self._post_event(0, format)


WlShm._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlSeat
from ..wayland import WlSurface
from .zwp_keyboard_shortcuts_inhibitor_v1 import ZwpKeyboardShortcutsInhibitorV1

import enum


class ZwpKeyboardShortcutsInhibitManagerV1(Interface):
    """Context object for keyboard grab_manager

    A global interface used for inhibiting the compositor keyboard shortcuts.
    """
    name = "zwp_keyboard_shortcuts_inhibit_manager_v1"
    version = 1

    error = enum.Enum("error", {
        "already_inhibited": 0,
    })


@ZwpKeyboardShortcutsInhibitManagerV1.request("", [])
def destroy(self):
    """Destroy the keyboard shortcuts inhibitor object

    Destroy the keyboard shortcuts inhibitor manager.
    """
    self._marshal(0)
    self._destroy()


@ZwpKeyboardShortcutsInhibitManagerV1.request("noo", [ZwpKeyboardShortcutsInhibitorV1, WlSurface, WlSeat])
def inhibit_shortcuts(self, surface, seat):
    """Create a new keyboard shortcuts inhibitor object

    Create a new keyboard shortcuts inhibitor object associated with the given
    surface for the given seat.

    If shortcuts are already inhibited for the specified seat and surface, a
    protocol error "already_inhibited" is raised by the compositor.

    :param surface:
        the surface that inhibits the keyboard shortcuts behavior
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    :param seat:
        the :class:`~pywayland.protocol.wayland.WlSeat` for which keyboard
        shortcuts should be disabled
    :type seat: :class:`~pywayland.protocol.wayland.WlSeat`
    :returns:
        :class:`~pywayland.protocol.keyboard_shortcuts_inhibit_unstable_v1.ZwpKeyboardShortcutsInhibitorV1`
    """
    id = self._marshal_constructor(1, ZwpKeyboardShortcutsInhibitorV1, surface, seat)
    return id


ZwpKeyboardShortcutsInhibitManagerV1._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2012, 2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlOutput

import enum


class ZwpInputPanelSurfaceV1(Interface):
    name = "zwp_input_panel_surface_v1"
    version = 1

    position = enum.Enum("position", {
        "center_bottom": 0,
    })


@ZwpInputPanelSurfaceV1.request("ou", [WlOutput, None])
def set_toplevel(self, output, position):
    """Set the surface type as a keyboard

    Set the input_panel_surface type to keyboard.

    A keyboard surface is only shown when a text input is active.

    :param output:
    :type output: :class:`~pywayland.protocol.wayland.WlOutput`
    :param position:
    :type position: `uint`
    """
    self._marshal(0, output, position)


@ZwpInputPanelSurfaceV1.request("", [])
def set_overlay_panel(self):
    """Set the surface type as an overlay panel

    Set the input_panel_surface to be an overlay panel.

    This is shown near the input cursor above the application window when a
    text input is active.
    """
    self._marshal(1)


ZwpInputPanelSurfaceV1._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlSurface
from .zxdg_positioner_v6 import ZxdgPositionerV6
from .zxdg_surface_v6 import ZxdgSurfaceV6

import enum


class ZxdgShellV6(Interface):
    """Create desktop-style surfaces

    :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell` allows clients
    to turn a :class:`~pywayland.protocol.wayland.WlSurface` into a "real
    window" which can be dragged, resized, stacked, and moved around by the
    user. Everything about this interface is suited towards traditional desktop
    environments.
    """
    name = "zxdg_shell_v6"
    version = 1

    error = enum.Enum("error", {
        "role": 0,
        "defunct_surfaces": 1,
        "not_the_topmost_popup": 2,
        "invalid_popup_parent": 3,
        "invalid_surface_state": 4,
        "invalid_positioner": 5,
    })


@ZxdgShellV6.request("", [])
def destroy(self):
    """Destroy :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell`

    Destroy this :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell`
    object.

    Destroying a bound
    :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell` object while
    there are surfaces still alive created by this
    :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell` object instance
    is illegal and will result in a protocol error.
    """
    self._marshal(0)
    self._destroy()


@ZxdgShellV6.request("n", [ZxdgPositionerV6])
def create_positioner(self):
    """Create a positioner object

    Create a positioner object. A positioner object is used to position
    surfaces relative to some parent surface. See the interface description and
    :func:`XdgSurface.get_popup()
    <pywayland.protocol.xdg_shell.XdgSurface..get_popup>` for details.

    :returns:
        :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgPositionerV6`
    """
    id = self._marshal_constructor(1, ZxdgPositionerV6)
    return id


@ZxdgShellV6.request("no", [ZxdgSurfaceV6, WlSurface])
def get_xdg_surface(self, surface):
    """Create a shell surface from a surface

    This creates an :class:`~pywayland.protocol.xdg_shell.XdgSurface` for the
    given surface. While :class:`~pywayland.protocol.xdg_shell.XdgSurface`
    itself is not a role, the corresponding surface may only be assigned a role
    extending :class:`~pywayland.protocol.xdg_shell.XdgSurface`, such as
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` or
    :class:`~pywayland.protocol.xdg_shell.XdgPopup`.

    This creates an :class:`~pywayland.protocol.xdg_shell.XdgSurface` for the
    given surface. An :class:`~pywayland.protocol.xdg_shell.XdgSurface` is used
    as basis to define a role to a given surface, such as
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` or
    :class:`~pywayland.protocol.xdg_shell.XdgPopup`. It also manages
    functionality shared between
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` based surface roles.

    See the documentation of :class:`~pywayland.protocol.xdg_shell.XdgSurface`
    for more details about what an
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` is and how it is used.

    :param surface:
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    :returns:
        :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgSurfaceV6`
    """
    id = self._marshal_constructor(2, ZxdgSurfaceV6, surface)
    return id


@ZxdgShellV6.request("u", [None])
def pong(self, serial):
    """Respond to a ping event

    A client must respond to a ping event with a pong request or the client may
    be deemed unresponsive. See :func:`XdgShell.ping()
    <pywayland.protocol.xdg_shell_unstable_v5.XdgShell..ping>`.

    :param serial:
        serial of the ping event
    :type serial: `uint`
    """
    self._marshal(3, serial)


@ZxdgShellV6.event("u", [None])
def ping(self, serial):
    """Check if the client is alive

    The ping event asks the client if it's still alive. Pass the serial
    specified in the event back to the compositor by sending a "pong" request
    back with the specified serial. See :func:`XdgShell.ping()
    <pywayland.protocol.xdg_shell_unstable_v5.XdgShell..ping>`.

    Compositors can use this to determine if the client is still alive. It's
    unspecified what will happen if the client doesn't respond to the ping
    request, or in what timeframe. Clients should try to respond in a
    reasonable amount of time.

    A compositor is free to ping in any way it wants, but a client must always
    respond to any :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell`
    object it created.

    :param serial:
        pass this to the pong request
    :type serial: `uint`
    """
    self._post_event(0, serial)


ZxdgShellV6._gen_c()



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .element import Element, Attribute

NO_IFACE_NAME = 'interface'


class Argument(Element):
    """Argument to a request or event method

    Required attributes: `name` and `type`

    Optional attributes: `summary`, `interface`, and `allow-null`

    Child elements: `description`
    """
    attributes = [
        Attribute('name', True),
        Attribute('type', True),
        Attribute('summary', False),
        Attribute('interface', False),
        Attribute('allow-null', False),
        Attribute('enum', False)
    ]

    @property
    def interface_class(self):
        """Returns the Interface class name

        Gives the class name for the Interface coresponding to the type of the
        argument.
        """
        return ''.join(x.capitalize() for x in self.interface.split('_'))

    @property
    def signature(self):
        """Return the signature of the argument

        Return the string corresponding to the signature of the argument as it
        appears in the signature of the wl_message struct.
        """
        if self.allow_null:
            return '?' + self.type_to_string()
        else:
            return self.type_to_string()

    def type_to_string(self):
        """Translate type to signature string"""
        if self.type == 'int':
            return 'i'
        elif self.type == 'uint':
            return 'u'
        elif self.type == 'fixed':
            return 'f'
        elif self.type == 'string':
            return 's'
        elif self.type == 'object':
            return 'o'
        elif self.type == 'new_id':
            if self.interface:
                return 'n'
            else:
                return 'sun'
        elif self.type == 'array':
            return 'a'
        elif self.type == 'fd':
            return 'h'

    def output_doc_param(self, printer):
        """Document the argument as a parameter"""
        # Output the parameter and summary
        printer.doc(':param {}:'.format(self.name))
        if self.summary:
            with printer.indented():
                printer.docstring(self.summary)

        # Determine the type to be output
        if self.interface:
            arg_type = self.interface
        else:
            arg_type = '`{}`'.format(self.type)

        # Output the parameter type
        if self.allow_null:
            printer.doc(':type {}: {} or `None`'.format(self.name, arg_type))
        else:
            printer.doc(':type {}: {}'.format(self.name, arg_type))

    def output_doc_ret(self, printer):
        """Document the argument as a return"""
        # Determine the type to be output
        if self.interface:
            arg_type = self.interface
        else:
            # Only new_id's are returned, the only corner case here is for
            # wl_registry.bind, so no interface => Proxy
            arg_type = ':class:`pywayland.client.proxy.Proxy` of specified Interface'

        # Output the type and summary
        printer.doc(':returns:')
        with printer.indented():
            if self.summary:
                printer.docstring('{} -- {}'.format(arg_type, self.summary))
            else:
                printer.docstring('{}'.format(arg_type))
